import { atom } from 'jotai';
import { QueryClient, QueryObserver } from 'react-query';

const queryClientAtom = atom(null);
const getQueryClient = (get, set) => {
  let queryClient = get(queryClientAtom);
  if (queryClient === null) {
    queryClient = new QueryClient();
    set(queryClientAtom, queryClient);
  }
  return queryClient;
};

function atomWithQuery(createQuery, equalityFn = Object.is) {
  const dataAtom = atom(new Promise(() => {
  }));
  const queryAtom = atom((get) => {
    const options = typeof createQuery === "function" ? createQuery(get) : createQuery;
    const observerAtom = atom(null, (get2, set, action) => {
      if (action.type === "init") {
        set(dataAtom, new Promise(() => {
        }));
        action.initializer(getQueryClient(get2, set));
      } else if (action.type === "data") {
        const data = get2(dataAtom);
        if (data === null || !equalityFn(data, action.data)) {
          set(dataAtom, action.data);
        }
      }
    });
    observerAtom.onMount = (dispatch) => {
      let unsub;
      const initializer = (queryClient) => {
        const observer = new QueryObserver(queryClient, options);
        observer.subscribe((result) => {
          if (result.data !== void 0) {
            dispatch({type: "data", data: result.data});
          }
        });
        if (unsub === false) {
          observer.destroy();
        } else {
          unsub = () => {
            observer.destroy();
          };
        }
      };
      dispatch({type: "init", initializer});
      return () => {
        if (unsub) {
          unsub();
        }
        unsub = false;
      };
    };
    return [options, observerAtom];
  }, (get, set, action) => {
    var _a;
    if (action.type === "refetch") {
      const [options] = get(queryAtom);
      set(dataAtom, new Promise(() => {
      }));
      const queryClient = getQueryClient(get, set);
      (_a = queryClient.getQueryCache().find(options.queryKey)) == null ? void 0 : _a.reset();
      const p = queryClient.refetchQueries(options.queryKey);
      return p;
    }
    return;
  });
  const queryDataAtom = atom((get) => {
    const [, observerAtom] = get(queryAtom);
    get(observerAtom);
    return get(dataAtom);
  }, (_get, set, action) => set(queryAtom, action));
  return queryDataAtom;
}

export { atomWithQuery, queryClientAtom };
