'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var jotai = require('jotai');
var reactQuery = require('react-query');

var queryClientAtom = jotai.atom(null);
var getQueryClient = function getQueryClient(get, set) {
  var queryClient = get(queryClientAtom);

  if (queryClient === null) {
    queryClient = new reactQuery.QueryClient();
    set(queryClientAtom, queryClient);
  }

  return queryClient;
};

function atomWithQuery(createQuery, equalityFn) {
  if (equalityFn === void 0) {
    equalityFn = Object.is;
  }

  var dataAtom = jotai.atom(new Promise(function () {}));
  var queryAtom = jotai.atom(function (get) {
    var options = typeof createQuery === 'function' ? createQuery(get) : createQuery;
    var observerAtom = jotai.atom(null, function (get, set, action) {
      if (action.type === 'init') {
        set(dataAtom, new Promise(function () {}));
        action.initializer(getQueryClient(get, set));
      } else if (action.type === 'data') {
        var data = get(dataAtom);

        if (data === null || !equalityFn(data, action.data)) {
          set(dataAtom, action.data);
        }
      }
    });

    observerAtom.onMount = function (dispatch) {
      var unsub;

      var initializer = function initializer(queryClient) {
        var observer = new reactQuery.QueryObserver(queryClient, options);
        observer.subscribe(function (result) {
          if (result.data !== undefined) {
            dispatch({
              type: 'data',
              data: result.data
            });
          }
        });

        if (unsub === false) {
          observer.destroy();
        } else {
          unsub = function unsub() {
            observer.destroy();
          };
        }
      };

      dispatch({
        type: 'init',
        initializer: initializer
      });
      return function () {
        if (unsub) {
          unsub();
        }

        unsub = false;
      };
    };

    return [options, observerAtom];
  }, function (get, set, action) {
    if (action.type === 'refetch') {
      var _queryClient$getQuery;

      var _get2 = get(queryAtom),
          options = _get2[0];

      set(dataAtom, new Promise(function () {}));

      var _queryClient = getQueryClient(get, set);

      (_queryClient$getQuery = _queryClient.getQueryCache().find(options.queryKey)) == null ? void 0 : _queryClient$getQuery.reset();

      var p = _queryClient.refetchQueries(options.queryKey);

      return p;
    }

    return;
  });
  var queryDataAtom = jotai.atom(function (get) {
    var _get3 = get(queryAtom),
        observerAtom = _get3[1];

    get(observerAtom);
    return get(dataAtom);
  }, function (_get, set, action) {
    return set(queryAtom, action);
  });
  return queryDataAtom;
}

exports.atomWithQuery = atomWithQuery;
exports.queryClientAtom = queryClientAtom;
